@@ -45,8 +45,8 @@ struct NautilusDesktopLinkDetails {
 
 	NautilusDesktopIconFile *icon_file;
 	
-	/* Just for trash icons: */
-	gulong trash_state_handler;
+	GObject *signal_handler_obj;
+	gulong signal_handler;
 
 	/* Just for mount icons: */
 	GMount *mount;
@@ -68,6 +68,24 @@ nautilus_desktop_link_changed (NautilusDesktopLink *link)
 	}
 }
 
+static void
+mount_changed_callback (GMount *mount, NautilusDesktopLink *link)
+{
+	g_free (link->details->display_name);
+	if (link->details->activation_location) {
+		g_object_unref (link->details->activation_location);
+	}
+	if (link->details->icon) {
+		g_object_unref (link->details->icon);
+	}
+	
+	link->details->display_name = g_mount_get_name (mount);
+	link->details->activation_location = g_mount_get_default_location (mount);
+	link->details->icon = g_mount_get_icon (mount);
+	
+	nautilus_desktop_link_changed (link);
+}
+
 static void
 trash_state_changed_callback (NautilusTrashMonitor *trash_monitor,
 			      gboolean state,
@@ -183,7 +201,8 @@ nautilus_desktop_link_new (NautilusDesktopLinkType type)
 		eel_preferences_add_callback (NAUTILUS_PREFERENCES_DESKTOP_TRASH_NAME,
 					      trash_name_changed,
 					      link);
-		link->details->trash_state_handler =
+		link->details->signal_handler_obj = G_OBJECT (nautilus_trash_monitor_get ());
+		link->details->signal_handler =
 			g_signal_connect_object (nautilus_trash_monitor_get (), "trash_state_changed",
 						 G_CALLBACK (trash_state_changed_callback), link, 0);	
 		break;
@@ -242,9 +261,14 @@ nautilus_desktop_link_new_from_mount (GMount *mount)
 	
 	link->details->display_name = g_mount_get_name (mount);
 	
-	link->details->activation_location = g_mount_get_root (mount);
+	link->details->activation_location = g_mount_get_default_location (mount);
 	link->details->icon = g_mount_get_icon (mount);
 	
+	link->details->signal_handler_obj = G_OBJECT (mount);
+	link->details->signal_handler =
+		g_signal_connect (mount, "changed",
+				  G_CALLBACK (mount_changed_callback), link);
+	
 	create_icon_file (link);
 
 	return link;
@@ -295,6 +319,16 @@ nautilus_desktop_link_get_activation_location (NautilusDesktopLink *link)
 	return NULL;
 }
 
+char *
+nautilus_desktop_link_get_activation_uri (NautilusDesktopLink *link)
+{
+	if (link->details->activation_location) {
+		return g_file_get_uri (link->details->activation_location);
+	}
+	return NULL;
+}
+
+
 gboolean
 nautilus_desktop_link_get_date (NautilusDesktopLink *link,
 				NautilusDateType     date_type,
@@ -358,9 +392,9 @@ desktop_link_finalize (GObject *object)
 
 	link = NAUTILUS_DESKTOP_LINK (object);
 
-	if (link->details->trash_state_handler != 0) {
-		g_signal_handler_disconnect (nautilus_trash_monitor_get (),
-					     link->details->trash_state_handler);
+	if (link->details->signal_handler != 0) {
+		g_signal_handler_disconnect (link->details->signal_handler_obj,
+					     link->details->signal_handler);
 	}
 
 	if (link->details->icon_file != NULL) {
