@@ -265,12 +265,41 @@ add_place (NautilusPlacesSidebar *sidebar,
 	return child_iter;
 }
 
+static void
+compare_for_selection (NautilusPlacesSidebar *sidebar,
+		       const gchar *location,
+		       const gchar *added_uri,
+		       const gchar *last_uri,
+		       GtkTreeIter *iter,
+		       GtkTreePath **path)
+{
+	int res;
+
+	res = eel_strcmp (added_uri, last_uri);
+
+	if (res == 0) {
+		/* last_uri always comes first */
+		if (*path != NULL) {
+			gtk_tree_path_free (*path);
+		}
+		*path = gtk_tree_model_get_path (sidebar->filter_model,
+						 iter);
+	} else if (eel_strcmp (location, added_uri) == 0) {
+		if (*path == NULL) {
+			*path = gtk_tree_model_get_path (sidebar->filter_model,
+							 iter);
+		}
+	}
+}
+
 static void
 update_places (NautilusPlacesSidebar *sidebar)
 {
 	NautilusBookmark *bookmark;
 	GtkTreeSelection *selection;
 	GtkTreeIter iter, last_iter;
+	GtkTreePath *select_path;
+	GtkTreeModel *model;
 	GVolumeMonitor *volume_monitor;
 	GList *mounts, *l, *ll;
 	GMount *mount;
@@ -279,13 +308,22 @@ update_places (NautilusPlacesSidebar *sidebar)
 	GList *volumes;
 	GVolume *volume;
 	int bookmark_count, index;
-	char *location, *mount_uri, *name, *desktop_path;
+	char *location, *mount_uri, *name, *desktop_path, *last_uri;
 	GIcon *icon;
 	GFile *root;
 	NautilusWindowSlotInfo *slot;
 	char *tooltip;
 
+	model = NULL;
+	last_uri = NULL;
+	select_path = NULL;
+
 	selection = gtk_tree_view_get_selection (sidebar->tree_view);
+	if (gtk_tree_selection_get_selected (selection, &model, &last_iter)) {
+		gtk_tree_model_get (model,
+				    &last_iter,
+				    PLACES_SIDEBAR_COLUMN_URI, &last_uri, -1);
+	}
 	gtk_list_store_clear (sidebar->store);
 
 	slot = nautilus_window_info_get_active_slot (sidebar->window);
@@ -306,9 +344,9 @@ update_places (NautilusPlacesSidebar *sidebar)
 				       _("Open your personal folder"));
 		g_object_unref (icon);
 		g_free (display_name);
-		if (eel_strcmp (location, mount_uri) == 0) {
-			gtk_tree_selection_select_iter (selection, &last_iter);
-		}	
+		compare_for_selection (sidebar,
+				       location, mount_uri, last_uri,
+				       &last_iter, &select_path);
 		g_free (mount_uri);
 	}
 
@@ -319,9 +357,9 @@ update_places (NautilusPlacesSidebar *sidebar)
 			       mount_uri, NULL, NULL, NULL, 0,
 			       _("Open the contents of your desktop in a folder"));
 	g_object_unref (icon);
-	if (eel_strcmp (location, mount_uri) == 0) {
-		gtk_tree_selection_select_iter (selection, &last_iter);
-	}	
+	compare_for_selection (sidebar,
+			       location, mount_uri, last_uri,
+			       &last_iter, &select_path);
 	g_free (mount_uri);
 	g_free (desktop_path);
 	
@@ -332,9 +370,9 @@ update_places (NautilusPlacesSidebar *sidebar)
 			       mount_uri, NULL, NULL, NULL, 0,
 			       _("Open the contents of the File System"));
 	g_object_unref (icon);
-	if (eel_strcmp (location, mount_uri) == 0) {
-		gtk_tree_selection_select_iter (selection, &last_iter);
-	}
+	compare_for_selection (sidebar,
+			       location, mount_uri, last_uri,
+			       &last_iter, &select_path);
 
  	mount_uri = "network:///"; /* No need to strdup */
 	icon = g_themed_icon_new (NAUTILUS_ICON_NETWORK);
@@ -343,9 +381,9 @@ update_places (NautilusPlacesSidebar *sidebar)
 			       mount_uri, NULL, NULL, NULL, 0,
 			       _("Browse the contents of the network"));
 	g_object_unref (icon);
-	if (strcmp (location, mount_uri) == 0) {
-		gtk_tree_selection_select_iter (selection, &last_iter);
-	}
+	compare_for_selection (sidebar,
+			       location, mount_uri, last_uri,
+			       &last_iter, &select_path);
 
 	volume_monitor = sidebar->volume_monitor;
 
@@ -369,9 +407,9 @@ update_places (NautilusPlacesSidebar *sidebar)
 					last_iter = add_place (sidebar, PLACES_MOUNTED_VOLUME,
 							       name, icon, mount_uri,
 							       drive, volume, mount, 0, tooltip);
-					if (eel_strcmp (location, mount_uri) == 0) {
-						gtk_tree_selection_select_iter (selection, &last_iter);
-					}
+					compare_for_selection (sidebar,
+							       location, mount_uri, last_uri,
+							       &last_iter, &select_path);
 					g_object_unref (root);
 					g_object_unref (mount);
 					g_object_unref (icon);
@@ -446,9 +484,9 @@ update_places (NautilusPlacesSidebar *sidebar)
 			last_iter = add_place (sidebar, PLACES_MOUNTED_VOLUME,
 					       name, icon, mount_uri,
 					       NULL, volume, mount, 0, tooltip);
-			if (eel_strcmp (location, mount_uri) == 0) {
-				gtk_tree_selection_select_iter (selection, &last_iter);
-			}
+			compare_for_selection (sidebar,
+					       location, mount_uri, last_uri,
+					       &last_iter, &select_path);
 			g_object_unref (mount);
 			g_object_unref (icon);
 			g_free (name);
@@ -490,9 +528,9 @@ update_places (NautilusPlacesSidebar *sidebar)
 		last_iter = add_place (sidebar, PLACES_MOUNTED_VOLUME,
 				       name, icon, mount_uri,
 				       NULL, NULL, mount, 0, tooltip);
-		if (eel_strcmp (location, mount_uri) == 0) {
-			gtk_tree_selection_select_iter (selection, &last_iter);
-		}
+		compare_for_selection (sidebar,
+				       location, mount_uri, last_uri,
+				       &last_iter, &select_path);
 		g_object_unref (root);
 		g_object_unref (mount);
 		g_object_unref (icon);
@@ -508,9 +546,9 @@ update_places (NautilusPlacesSidebar *sidebar)
 			       _("Trash"), icon, mount_uri,
 			       NULL, NULL, NULL, 0,
 			       _("Open the trash"));
-	if (eel_strcmp (location, mount_uri) == 0) {
-		gtk_tree_selection_select_iter (selection, &last_iter);
-	}
+	compare_for_selection (sidebar,
+			       location, mount_uri, last_uri,
+			       &last_iter, &select_path);
 	g_object_unref (icon);
 
 	/* add separator */
@@ -539,9 +577,9 @@ update_places (NautilusPlacesSidebar *sidebar)
 				       name, icon, mount_uri,
 				       NULL, NULL, NULL, index, 
 				       tooltip);
-		if (eel_strcmp (location, mount_uri) == 0) {
-			gtk_tree_selection_select_iter (selection, &last_iter);
-		}
+		compare_for_selection (sidebar,
+				       location, mount_uri, last_uri,
+				       &last_iter, &select_path);
 		g_free (name);
 		g_object_unref (root);
 		g_object_unref (icon);
@@ -549,6 +587,16 @@ update_places (NautilusPlacesSidebar *sidebar)
 		g_free (tooltip);
 	}
 	g_free (location);
+
+	if (select_path != NULL) {
+		gtk_tree_selection_select_path (selection, select_path);
+	}
+
+	if (select_path != NULL) {
+		gtk_tree_path_free (select_path);
+	}
+
+	g_free (last_uri);
 }
 
 static gboolean
@@ -684,8 +732,6 @@ clicked_eject_button (NautilusPlacesSidebar *sidebar,
 		    button_event->x - total_width <= eject_button_size) {
 			return TRUE;
 		}
-	} else if (event->type == GDK_BUTTON_RELEASE) {
-		g_object_set_data (G_OBJECT (sidebar->tree_view), "myrect", NULL);
 	}
 
 	if (*path != NULL) {
@@ -695,18 +741,6 @@ clicked_eject_button (NautilusPlacesSidebar *sidebar,
 	return FALSE;
 }
 
-static void
-row_activated_callback (GtkTreeView *tree_view,
-			GtkTreePath *path,
-			GtkTreeViewColumn *column,
-			gpointer user_data)
-{
-	open_selected_bookmark (NAUTILUS_PLACES_SIDEBAR (user_data),
-				gtk_tree_view_get_model (tree_view),
-				path,
-				0);
-}
-
 static void
 desktop_location_changed_callback (gpointer user_data)
 {
@@ -846,7 +880,7 @@ get_drag_data (GtkTreeView *tree_view,
 	       unsigned int time)
 {
 	GdkAtom target;
-	
+
 	target = gtk_drag_dest_find_target (GTK_WIDGET (tree_view), 
 					    context, 
 					    NULL);
@@ -901,14 +935,6 @@ can_accept_items_as_bookmarks (const GList *items)
 	return TRUE;
 }
 
-static void
-drag_data_delete_callback (GtkWidget             *widget,
-			   GdkDragContext        *context,
-			   NautilusPlacesSidebar *sidebar)
-{
-	g_signal_stop_emission_by_name (widget, "drag-data-delete");
-}
-
 static gboolean
 drag_motion_callback (GtkTreeView *tree_view,
 		      GdkDragContext *context,
@@ -967,10 +993,11 @@ drag_motion_callback (GtkTreeView *tree_view,
 
 	if (action != 0) {
 		gdk_drag_status (context, action, time);
-		return TRUE;
 	} else {
-		return FALSE;
+		gdk_drag_status (context, 0, time);
 	}
+
+	return TRUE;
 }
 
 static void
@@ -1099,7 +1126,6 @@ reorder_bookmarks (NautilusPlacesSidebar *sidebar,
 		   int                   new_position)
 {
 	GtkTreeIter iter;
-	NautilusBookmark *bookmark;
 	PlaceType type; 
 	int old_position;
 
@@ -1119,17 +1145,8 @@ reorder_bookmarks (NautilusPlacesSidebar *sidebar,
 		return;
 	}
 
-	/* Remove the path from the old position and insert it in the new one */
-
-	if (old_position == new_position) {
-		return;
-	}
-	bookmark = nautilus_bookmark_list_item_at (sidebar->bookmarks, old_position);
-	nautilus_bookmark_list_insert_item (sidebar->bookmarks, bookmark, new_position);
-	if (old_position > new_position) {
-		old_position++;
-	}
-	nautilus_bookmark_list_delete_item_at (sidebar->bookmarks, old_position);
+	nautilus_bookmark_list_move_item (sidebar->bookmarks, old_position,
+					  new_position);
 }
 
 static void
@@ -1317,7 +1334,7 @@ check_unmount_and_eject (GMount *mount,
 	}
 	if (mount != NULL) {
 		*show_eject |= g_mount_can_eject (mount);
-		*show_unmount = g_mount_can_unmount (mount);
+		*show_unmount = g_mount_can_unmount (mount) && !*show_eject;
 	}
 }
 
@@ -1349,19 +1366,15 @@ check_visibility (GMount           *mount,
 
 		*show_start = g_drive_can_start (drive) || g_drive_can_start_degraded (drive);
 		*show_stop  = g_drive_can_stop (drive);
+
+		if (*show_stop)
+			*show_unmount = FALSE;
 	}
 
 	if (volume != NULL) {
 		if (mount == NULL)
 			*show_mount = g_volume_can_mount (volume);
 	}
-
-#ifdef TODO_GIO
-		if (something &&
-		    g_find_program_in_path ("gfloppy")) {
-			*show_format = TRUE;
-		}
-#endif		
 }
 
 static void
@@ -1737,12 +1750,25 @@ mount_shortcut_cb (GtkMenuItem           *item,
 	}
 }
 
+static void
+unmount_done (gpointer data)
+{
+	NautilusWindow *window;
+
+	window = data;
+	nautilus_window_info_set_initiated_unmount (window, FALSE);
+	g_object_unref (window);
+}
+
 static void
 do_unmount (GMount *mount,
 	    NautilusPlacesSidebar *sidebar)
 {
 	if (mount != NULL) {
-		nautilus_file_operations_unmount_mount (NULL, mount, FALSE, TRUE);
+		nautilus_window_info_set_initiated_unmount (sidebar->window, TRUE);
+		nautilus_file_operations_unmount_mount_full (NULL, mount, FALSE, TRUE,
+							     unmount_done,
+							     g_object_ref (sidebar->window));
 	}
 }
 
@@ -1778,9 +1804,15 @@ drive_eject_cb (GObject *source_object,
 		GAsyncResult *res,
 		gpointer user_data)
 {
+	NautilusWindow *window;
 	GError *error;
 	char *primary;
 	char *name;
+
+	window = user_data;
+	nautilus_window_info_set_initiated_unmount (window, FALSE);
+	g_object_unref (window);
+
 	error = NULL;
 	if (!g_drive_eject_with_operation_finish (G_DRIVE (source_object), res, &error)) {
 		if (error->code != G_IO_ERROR_FAILED_HANDLED) {
@@ -1801,9 +1833,15 @@ volume_eject_cb (GObject *source_object,
 		GAsyncResult *res,
 		gpointer user_data)
 {
+	NautilusWindow *window;
 	GError *error;
 	char *primary;
 	char *name;
+
+	window = user_data;
+	nautilus_window_info_set_initiated_unmount (window, FALSE);
+	g_object_unref (window);
+
 	error = NULL;
 	if (!g_volume_eject_with_operation_finish (G_VOLUME (source_object), res, &error)) {
 		if (error->code != G_IO_ERROR_FAILED_HANDLED) {
@@ -1824,9 +1862,15 @@ mount_eject_cb (GObject *source_object,
 		GAsyncResult *res,
 		gpointer user_data)
 {
+	NautilusWindow *window;
 	GError *error;
 	char *primary;
 	char *name;
+
+	window = user_data;
+	nautilus_window_info_set_initiated_unmount (window, FALSE);
+	g_object_unref (window);
+
 	error = NULL;
 	if (!g_mount_eject_with_operation_finish (G_MOUNT (source_object), res, &error)) {
 		if (error->code != G_IO_ERROR_FAILED_HANDLED) {
@@ -1852,11 +1896,17 @@ do_eject (GMount *mount,
 
 	mount_op = gtk_mount_operation_new (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (sidebar))));
 	if (mount != NULL) {
-		g_mount_eject_with_operation (mount, 0, mount_op, NULL, mount_eject_cb, NULL);
+		nautilus_window_info_set_initiated_unmount (sidebar->window, TRUE);
+		g_mount_eject_with_operation (mount, 0, mount_op, NULL, mount_eject_cb,
+					      g_object_ref (sidebar->window));
 	} else if (volume != NULL) {
-		g_volume_eject_with_operation (volume, 0, mount_op, NULL, volume_eject_cb, NULL);
+		nautilus_window_info_set_initiated_unmount (sidebar->window, TRUE);
+		g_volume_eject_with_operation (volume, 0, mount_op, NULL, volume_eject_cb,
+					      g_object_ref (sidebar->window));
 	} else if (drive != NULL) {
-		g_drive_eject_with_operation (drive, 0, mount_op, NULL, drive_eject_cb, NULL);
+		nautilus_window_info_set_initiated_unmount (sidebar->window, TRUE);
+		g_drive_eject_with_operation (drive, 0, mount_op, NULL, drive_eject_cb,
+					      g_object_ref (sidebar->window));
 	}
 	g_object_unref (mount_op);
 }
@@ -2063,10 +2113,15 @@ drive_stop_cb (GObject *source_object,
 	       GAsyncResult *res,
 	       gpointer user_data)
 {
+	NautilusWindow *window;
 	GError *error;
 	char *primary;
 	char *name;
 
+	window = user_data;
+	nautilus_window_info_set_initiated_unmount (window, FALSE);
+	g_object_unref (window);
+
 	error = NULL;
 	if (!g_drive_poll_for_media_finish (G_DRIVE (source_object), res, &error)) {
 		if (error->code != G_IO_ERROR_FAILED_HANDLED) {
@@ -2101,7 +2156,9 @@ stop_shortcut_cb (GtkMenuItem           *item,
 		GMountOperation *mount_op;
 
 		mount_op = gtk_mount_operation_new (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (sidebar))));
-		g_drive_stop (drive, G_MOUNT_UNMOUNT_NONE, mount_op, NULL, drive_stop_cb, NULL);
+		nautilus_window_info_set_initiated_unmount (sidebar->window, TRUE);
+		g_drive_stop (drive, G_MOUNT_UNMOUNT_NONE, mount_op, NULL, drive_stop_cb,
+			      g_object_ref (sidebar->window));
 		g_object_unref (mount_op);
 	}
 	g_object_unref (drive);
@@ -2129,8 +2186,7 @@ bookmarks_key_press_event_cb (GtkWidget             *widget,
       return eject_or_unmount_selection (sidebar);
   }
 
-  if ((event->keyval == GDK_BackSpace
-      || event->keyval == GDK_Delete
+  if ((event->keyval == GDK_Delete
       || event->keyval == GDK_KP_Delete)
       && (event->state & modifiers) == 0) {
       remove_selected_bookmarks (sidebar);
@@ -2292,6 +2348,39 @@ bookmarks_popup_menu_cb (GtkWidget *widget,
 	return TRUE;
 }
 
+static gboolean
+bookmarks_button_release_event_cb (GtkWidget *widget,
+				   GdkEventButton *event,
+				   NautilusPlacesSidebar *sidebar)
+{
+	GtkTreePath *path;
+	GtkTreeModel *model;
+	GtkTreeView *tree_view;
+
+	if (event->type != GDK_BUTTON_RELEASE) {
+		return TRUE;
+	}
+
+	tree_view = GTK_TREE_VIEW (widget);
+	model = gtk_tree_view_get_model (tree_view);
+
+	if (event->button == 1) {
+
+		if (event->window != gtk_tree_view_get_bin_window (tree_view)) {
+			return FALSE;
+		}
+
+		gtk_tree_view_get_path_at_pos (tree_view, (int) event->x, (int) event->y,
+					       &path, NULL, NULL, NULL);
+
+		open_selected_bookmark (sidebar, model, path, 0);
+
+		gtk_tree_path_free (path);
+	}
+
+	return FALSE;
+}
+
 /* Callback used when a button is pressed on the shortcuts list.  
  * We trap button 3 to bring up a popup menu, and button 2 to
  * open in a new tab.
@@ -2510,11 +2599,6 @@ nautilus_places_sidebar_init (NautilusPlacesSidebar *sidebar)
 	selection = gtk_tree_view_get_selection (tree_view);
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_BROWSE);
 
-	g_signal_connect_object
-		(tree_view, "row_activated", 
-		 G_CALLBACK (row_activated_callback), sidebar, 0);
-
-
 	gtk_tree_view_enable_model_drag_source (GTK_TREE_VIEW (tree_view),
 						GDK_BUTTON1_MASK,
 						nautilus_shortcuts_source_targets,
@@ -2528,8 +2612,6 @@ nautilus_places_sidebar_init (NautilusPlacesSidebar *sidebar)
 	g_signal_connect (tree_view, "key-press-event",
 			  G_CALLBACK (bookmarks_key_press_event_cb), sidebar);
 
-	g_signal_connect (tree_view, "drag-data-delete",
-			  G_CALLBACK (drag_data_delete_callback), sidebar);
 	g_signal_connect (tree_view, "drag-motion",
 			  G_CALLBACK (drag_motion_callback), sidebar);
 	g_signal_connect (tree_view, "drag-leave",
@@ -2544,6 +2626,8 @@ nautilus_places_sidebar_init (NautilusPlacesSidebar *sidebar)
 			  G_CALLBACK (bookmarks_popup_menu_cb), sidebar);
 	g_signal_connect (tree_view, "button-press-event",
 			  G_CALLBACK (bookmarks_button_press_event_cb), sidebar);
+	g_signal_connect (tree_view, "button-release-event",
+			  G_CALLBACK (bookmarks_button_release_event_cb), sidebar);
 
 	eel_gtk_tree_view_set_activate_on_single_click (sidebar->tree_view,
 							TRUE);
